@@ -1,3 +1,4 @@
+## -------------------------------------------------------------------
 #' Univariate Fourier integrals
 #'
 #' It computes Fourier integrals of functions of one and two
@@ -5,41 +6,52 @@
 #'
 #' See vignette for more detailed examples.
 #'
-#' @param f function or a vector of size m. If a function is provided,
-#'     it must be able to be evaluated in vectors. If a vector of
-#'     values is provided, such evaluations must have been obtained on
-#'     a regular grid and the Fourier integral is faster is m is a
-#'     power of 2.
-#' @param a Lower integration limit.
-#' @param b Upper integration limit.
-#' @param c Lower evaluation limit.
-#' @param d Upper evaluation limit.
-#' @param r Factor related to adjust definition of Fourier
+#' @param f function or a vector of size m. If a function is
+#'     provided, it must be able to be evaluated at vectors. If a
+#'     vector of values is provided, such evaluations must have been
+#'     obtained on a regular grid and the Fourier integral is faster
+#'     is m is a power of 2.
+#' @param lower_int Lower integration limit(s).
+#' @param upper_int Upper integration limit(s).
+#' @param lower_eval Lower evaluation limit(s). It can be NULL if an
+#'     evaluation grid is provided.
+#' @param upper_eval Upper evaluation limit(s). It can be NULL if an
+#'     evaluation grid is provided.
+#' @param const_adj Factor related to adjust definition of Fourier
 #'     transform. It is usually equal to 0, -1 or 1.
-#' @param s Constant to adjust the exponent on the definition of the
-#'     Fourier transform. It is usually equal to 1, -1, 2pi or -2pi.
-#' @param resol An integer (faster if power of two) determining the
-#'     resolution of the evaluation grid. Not required if f is a
-#'     vector.
-#' @param w An optional vector where the Fourier integral will be
-#'     evaluated. If provided, the FFT will NOT be used.
+#' @param freq_adj Constant to adjust the exponent on the definition
+#'     of the Fourier transform. It is usually equal to 1, -1, 2pi or
+#'     -2pi.
+#' @param resolution A vector of integers (faster if powers of two)
+#'     determining the resolution of the evaluation grid. Not
+#'     required if f is a vector.
+#' @param eval_grid Optional matrix with d columns with the points
+#'     where the Fourier integral will be evaluated. If it is
+#'     provided, the FFT will not be used.
 #' @param use_fft Logical value specifying whether the FFT will be
-#'     used to compute the Fourier integral.
+#'     used.
 #' @return If w is given, only the values of the Fourier integral are
 #'     returned, otherwise, a list with the elements \item{w}{A vector
 #'     of size m where the integral was computed.}  \item{values}{A
 #'     complex vector of size m with the values of the integral}
 #'
 #' @examples
-#' ##--- Example 1 ------------------------------------------------------
-#' ##--- Recovering std. normal from its characteristic function --------
+#' ##--- Example 1 ---------------------------------------------------
+#' ##--- Recovering std. normal from its characteristic function -----
 #' library(fourierin)
 #'
-#'                                         # Compute integral
-#' out <- fourierin_1d(f = function(t) exp(-t^2/2),
-#'                  a = -5, b = 5, c = -3, d = 3,
-#'                  r = -1, s = -1, resol = 64)
-#' grid <- out$w                           # Extract grid and values
+#' #' Function to to be used in integrand
+#' myfun <- function(t) exp(-t^2/2)
+#'
+#'                                         # Compute Foueien integral
+#' out <- fourierin_1d(f = myfun,
+#'                     lower_int = -5, upper_int = 5,
+#'                     lower_eval = -3, upper_eval = 3,
+#'                     const_adj = -1, freq_adj = -1,
+#'                     resolution = 64)
+#'
+#' ## Extract grid and values
+#' grid <- out$w
 #' values <- Re(out$values)
 #'
 #' plot(grid, values, type = "l", col = 3)
@@ -49,37 +61,57 @@
 #' ##--- Computing characteristic function of a gamma r. v. ------
 #'
 #' library(fourierin)
-#'                                         # Compute integral
+#'
+#' ## Function to to be used in integrand
+#' myfun <- function(t) dgamma(t, shape, rate)
+#'
+#' ## Compute integral
 #' shape <- 5
 #' rate <- 3
-#' out <- fourierin_1d(f = function(t) dgamma(t, shape, rate),
-#'                  a = 0, b = 6, c = -4, d = 4,
-#'                  r = 1, s = 1, resol = 64)
+#' out <- fourierin_1d(f = myfun, lower_int = 0, upper_int = 6,
+#'                     lower_eval = -4, upper_eval = 4,
+#'                     const_adj = 1, freq_adj = 1, resolution = 64)
+#'
 #' grid <- out$w                           # Extract grid
 #' re_values <- Re(out$values)             # Real values
 #' im_values <- Im(out$values)             # Imag values
 #'
-#'                                         # Now compute the real and
-#'                                         # imaginary true values of the
+#'                                      # Now compute the real and
+#'                                      # imaginary true values of the
 #'                                         # characteric function.
 #' true_cf <- function(t, shape, rate) (1 - 1i*t/rate)^-shape
 #' true_re <- Re(true_cf(grid, shape, rate))
 #' true_im <- Im(true_cf(grid, shape, rate))
 #'
-#'                                         # Compare them. We can see a
-#'                                         # slight discrepancy on the
-#'                                         # tails, but that is fixed
-#'                                         # when resulution is
-#'                                         # increased.
+#'                                      # Compare them. We can see a
+#'                                      # slight discrepancy on the
+#'                                      # tails, but that is fixed
+#'                                      # when resulution is
+#'                                      # increased.
 #' plot(grid, re_values, type = "l", col = 3)
 #' lines(grid, true_re, col = 4)
+#'
 #'                                         # Same here
 #' plot(grid, im_values, type = "l", col = 3)
 #' lines(grid, true_im, col = 4)
 #' @export
-fourierin_1d <- function(f, a, b, c = NULL, d = NULL,
-                         r, s, resol = NULL, w = NULL,
-                         use_fft = TRUE) {
+fourierin_1d <- function(f, lower_int, upper_int,
+                         lower_eval = NULL, upper_eval = NULL,
+                         const_adj, freq_adj, resolution = NULL,
+                         eval_grid = NULL, use_fft = TRUE) {
+    ## Condensed notation
+    a <- lower_int
+    b <- upper_int
+    c <- lower_eval
+    d <- upper_eval
+    r <- const_adj
+    s <- freq_adj
+    resol <- resolution
+    w <- eval_grid
+
+## fourierin_1d <- function(f, a, b, c = NULL, d = NULL,
+##                          r, s, resol = NULL, w = NULL,
+##                          use_fft = TRUE) {
     ## Flag to determine wheter a list or a vector will be returned
     w_given <- !is.null(w)
 
@@ -131,7 +163,7 @@ fourierin_1d <- function(f, a, b, c = NULL, d = NULL,
     if(w_given) return(out)
 
     return(list(w = w,                  # Return list.
-                values = out))
+                values = drop(out)))
 }
 
 #' Bivariate Fourier integrals
@@ -139,27 +171,7 @@ fourierin_1d <- function(f, a, b, c = NULL, d = NULL,
 #' It computes Fourier integrals for functions of one and two
 #' variables.
 #'
-#' @param f function or a matrix of size m1 x m2. If a function is
-#'     provided, it must be able to be evaluated in a matrix of two
-#'     columns. If a matrix of values is provided instead, such
-#'     evaluations must have been obtained on a regular grid matrix
-#'     and the Fourier integral is faster is m1 and m2 are powers of
-#'     2.
-#' @param a Lower integration limits.
-#' @param b Upper integration limits.
-#' @param c Lower evaluation limits.
-#' @param d Upper evaluation limits.
-#' @param r Factor related to adjust definition of Fourier
-#'     transform. It is usually 0, -1 or 1.
-#' @param s Constant to adjust the exponent on the definition of the
-#'     Fourier transform. It is usually 1, -1, 2pi or -2pi.
-#' @param resol A vector of two integers (faster if entries are powers
-#'     of two) determining the resolution of the evaluation grid. Not
-#'     required if f is a vector.
-#' @param w An optional two-column matrix where the Fourier integral
-#'     will be evaluated. If provided, the FFT will NOT be used.
-#' @param use_fft Logical value specifying whether the FFT will be
-#'     used to compute the Fourier integral.
+#' @inheritParams fourierin_1d
 #' @return If w is given, only the values of the Fourier integral are
 #'     returned, otherwise, a list with three elements
 #'     \item{w1}{Evaluation grid for first entry} \item{w2}{Evaluation
@@ -167,7 +179,7 @@ fourierin_1d <- function(f, a, b, c = NULL, d = NULL,
 #'     numbers, correspoding to the evaluations of the integral}
 #'
 #' @examples
-#' ##--- Recovering std. normal from its characteristic function --------
+#' ##--- Recovering std. normal from its characteristic function -----
 #' library(fourierin)
 #'
 #' ##-Parameters of bivariate normal distribution
@@ -196,9 +208,12 @@ fourierin_1d <- function(f, a, b, c = NULL, d = NULL,
 #' }
 #'
 #' ##-Approximate cf using Fourier integrals
-#' eval <- fourierin_2d(f, a = c(-8, -6), b = c(6, 8),
-#'                     c = c(-4, -4), d = c(4, 4),
-#'                     r = 1, s = 1, resol = c(128, 128))
+#' eval <- fourierin_2d(f, lower_int = c(-8, -6), upper_int = c(6, 8),
+#'                      lower_eval = c(-4, -4), upper_eval = c(4, 4),
+#'                      const_adj = 1, freq_adj =  1,
+#'                      resolution = c(128, 128))
+#'
+#' ## Extract values
 #' t1 <- eval$w1
 #' t2 <- eval$w2
 #' t <- as.matrix(expand.grid(t1 = t1, t2 = t2))
@@ -226,8 +241,21 @@ fourierin_1d <- function(f, a, b, c = NULL, d = NULL,
 #' legend("topleft", legend = c("true", "approximation"),
 #'        col = 3:2, lwd = 1)
 #' @export
-fourierin_2d <- function(f, a, b, c = NULL, d = NULL, r, s, resol = NULL,
-                         w = NULL, use_fft = TRUE){
+fourierin_2d <- function(f, lower_int, upper_int,
+                         lower_eval = NULL, upper_eval = NULL,
+                         const_adj, freq_adj,
+                         resolution = NULL, eval_grid = NULL,
+                         use_fft = TRUE){
+    ## Condensed notation
+    a <- lower_int
+    b <- upper_int
+    c <- lower_eval
+    d <- upper_eval
+    r <- const_adj
+    s <- freq_adj
+    resol <- resolution
+    w <- eval_grid
+
     ## If function values are provided, then the resolution is the
     ## length of the vector of values.
     if(!is.function(f)) resol <- dim(f)
@@ -305,25 +333,7 @@ fourierin_2d <- function(f, a, b, c = NULL, d = NULL, r, s, resol = NULL,
 #'
 #' See plenty of detailed examples in the vignette.
 #'
-#' @param f A function which can be evaluated in matrices of n columns
-#'     (n = 1 or n = 2). Or a matrix of n columns with f already
-#'     evaluated.
-#' @param a Lower integration limit(s).
-#' @param b Upper integration limit(s).
-#' @param c Lower evaluation limit(s).
-#' @param d Upper evaluation limit(s).
-#' @param r Factor related to adjust definition of Fourier
-#'     transform. It is usually 0, -1 or 1.
-#' @param s Constant to adjust the exponent on the definition of the
-#'     Fourier transform. It is usually 1, -1, 2pi or -2pi.
-#' @param resol An integer (faster if power of two) determining the
-#'     resolution of the evaluation grid. Not required if f is a
-#'     vector.
-#' @param w Optional matrix with d columns with the points where the
-#'     Fourier integral will be evaluated. If it is provided, the FFT
-#'     will not be used.
-#' @param use_fft Logical value specifying whether the FFT will be
-#'     used.
+#' @inheritParams fourierin_1d
 #' @return A list with the elements n-dimensional array and n vectors
 #'     with their corresponding resolution. Specifically,
 #'     \item{values}{A n-dimensional (resol_1 x resol_2 x ... x
@@ -331,46 +341,55 @@ fourierin_2d <- function(f, a, b, c = NULL, d = NULL, r, s, resol = NULL,
 #'     size resol_1} \item{...}{ } \item{wn}{A vector of size resol_n}
 #'
 #' @examples
-#' ##--- Example 1 ------------------------------------------------------
-#' ##--- Recovering std. normal from its characteristic function --------
+#' ##--- Example 1 ---------------------------------------------------
+#' ##--- Recovering std. normal from its characteristic function -----
 #' library(fourierin)
 #'
-#'                                         # Compute integral
-#' out <- fourierin(f = function(t) exp(-t^2/2),
-#'                  a = -5, b = 5, c = -3, d = 3,
-#'                  r = -1, s = -1, resol = 64)
-#' grid <- out$w                           # Extract grid and values
+#' ## Function to be used in the integrand
+#' myfnc <- function(t) exp(-t^2/2)
+#'
+#' ## Compute integral
+#' out <- fourierin(f = myfnc, lower_int = -5, upper_int = 5,
+#'                  lower_eval= -3, upper_eval = 3, const_adj = -1,
+#'                  freq_adj = -1, resolution = 64)
+#'
+#' ## Extract grid and values
+#' grid <- out$w
 #' values <- Re(out$values)
 #'
+#' ## Compare with true values of Fourier transform
 #' plot(grid, values, type = "l", col = 3)
 #' lines(grid, dnorm(grid), col = 4)
 #'
-#' ##--- Example 2 ------------------------------------------------------
-#' ##--- Computing characteristic function of a gamma r. v. ------------
+#'
+#' ##--- Example 2 ---------------------------------------------------
+#' ##--- Computing characteristic function of a gamma r. v. ----------
 #'
 #' library(fourierin)
-#'                                         # Compute integral
+#'
+#' ## Function to be used in integrand
+#' myfnc <- function(t) dgamma(t, shape, rate)
+#'
+#' ## Compute integral
 #' shape <- 5
 #' rate <- 3
-#' out <- fourierin(f = function(t) dgamma(t, shape, rate),
-#'                  a = 0, b = 6, c = -4, d = 4,
-#'                  r = 1, s = 1, resol = 64)
+#' out <- fourierin(f = myfnc, lower_int = 0, upper_int = 6,
+#'                  lower_eval = -4, upper_eval = 4,
+#'                  const_adj = 1, freq_adj = 1, resolution = 64)
+#'
+#' ## Extract values
 #' grid <- out$w                           # Extract grid
 #' re_values <- Re(out$values)             # Real values
 #' im_values <- Im(out$values)             # Imag values
 #'
-#'                                         # Now compute the real and
-#'                                         # imaginary true values of the
-#'                                         # characteric function.
+#' ## Now compute the real and imaginary true values of the
+#' ## characteric function.
 #' true_cf <- function(t, shape, rate) (1 - 1i*t/rate)^-shape
 #' true_re <- Re(true_cf(grid, shape, rate))
 #' true_im <- Im(true_cf(grid, shape, rate))
 #'
-#'                                         # Compare them. We can see a
-#'                                         # slight discrepancy on the
-#'                                         # tails, but that is fixed
-#'                                         # when resulution is
-#'                                         # increased.
+#' ## Compare them. We can see a slight discrepancy on the tails,
+#' ## but that is fixed when resulution is increased.
 #' plot(grid, re_values, type = "l", col = 3)
 #' lines(grid, true_re, col = 4)
 #'
@@ -378,8 +397,8 @@ fourierin_2d <- function(f, a, b, c = NULL, d = NULL, r, s, resol = NULL,
 #' plot(grid, im_values, type = "l", col = 3)
 #' lines(grid, true_im, col = 4)
 #'
-#' ##--- Example 3 ------------------------------------------------------
-#' ##--- Recovering std. normal from its characteristic function --------
+#' ##--- Example 3 -------------------------------------------------
+#' ##--- Recovering std. normal from its characteristic function ---
 #' library(fourierin)
 #'
 #' ##-Parameters of bivariate normal distribution
@@ -392,32 +411,34 @@ fourierin_2d <- function(f, a, b, c = NULL, d = NULL, r, s, resol = NULL,
 #'     ##-Auxiliar values
 #'     d <- ncol(x)
 #'     z <- sweep(x, 2, mu, "-")
-#'
 #'     ##-Get numerator and denominator of normal density
 #'     num <- exp(-0.5*rowSums(z * (z %*% solve(sig))))
 #'     denom <- sqrt((2*pi)^d*det(sig))
-#'
 #'     return(num/denom)
 #' }
 #'
-#' ##-Characteristic function
-#' ##-s is n x d
+#' ## Characteristic function
+#' ## s is n x d
 #' phi <- function(s) {
 #'     complex(modulus = exp(- 0.5*rowSums(s*(s %*% sig))),
 #'             argument = s %*% mu)
 #' }
 #'
 #' ##-Approximate cf using Fourier integrals
-#' eval <- fourierin(f, a = c(-8, -6), b = c(6, 8),
-#'                     c = c(-4, -4), d = c(4, 4),
-#'                     r = 1, s = 1, resol = c(128, 128))
+#' eval <- fourierin(f, lower_int = c(-8, -6), upper_int = c(6, 8),
+#'                   lower_eval = c(-4, -4), upper_eval = c(4, 4),
+#'                   const_adj = 1, freq_adj =  1,
+#'                   resolution = c(128, 128))
+#'
+#' ## Extract values
 #' t1 <- eval$w1
 #' t2 <- eval$w2
 #' t <- as.matrix(expand.grid(t1 = t1, t2 = t2))
 #' approx <- eval$values
 #' true <- matrix(phi(t), 128, 128)        # Compute true values
 #'
-#' ##-This is a section of the characteristic functions
+#'
+#' ## This is a section of the characteristic function
 #' i <- 65
 #' plot(t2, Re(approx[i, ]), type = "l", col = 2,
 #'      ylab = "",
@@ -438,19 +459,32 @@ fourierin_2d <- function(f, a, b, c = NULL, d = NULL, r, s, resol = NULL,
 #' legend("topleft", legend = c("true", "approximation"),
 #'        col = 3:2, lwd = 1)
 #' @export
-fourierin <- function(f, a, b, c = NULL, d = NULL, r, s,
-                      resol = NULL, w = NULL, use_fft = TRUE){
+fourierin <- function(f, lower_int, upper_int,
+                      lower_eval = NULL, upper_eval = NULL,
+                      const_adj, freq_adj,
+                      resolution = NULL, eval_grid = NULL,
+                      use_fft = TRUE){
 
-  n <- length(a)                      # Get dimension of function
-  # from lower integration
-  # limit.
-  switch(n,
-         return(fourierin_1d(f, a, b, c, d, r, s, resol, w, use_fft)),
-         return(fourierin_2d(f, a, b, c, d, r, s, resol, w, use_fft))
-  ) # End switch
+    ## Get dimension of function from lower integration limit.
+    n <- length(lower_int)
+
+    switch(n,
+           return(fourierin_1d(f, lower_int, upper_int,
+                               lower_eval, upper_eval,
+                               const_adj, freq_adj, resolution,
+                               eval_grid, use_fft)),
+           return(fourierin_2d(f, lower_int, upper_int,
+                               lower_eval, upper_eval,
+                               const_adj, freq_adj,
+                               resolution, eval_grid, use_fft))
+           ) # End switch
 }
 
-#' @useDynLib fourierin
+#' @useDynLib fourierin, .registration=TRUE
 #' @importFrom Rcpp sourceCpp
 NULL
 
+
+.onUnload <- function (libpath) {
+  library.dynam.unload("fourierin", libpath)
+}
